ProductionPointSellingFix = {}

function ProductionPointSellingFix.registerEventListeners(placeableType)
    SpecializationUtil.registerEventListener(placeableType, "onBuy", ProductionPointSellingFix)
    SpecializationUtil.registerEventListener(placeableType, "onSell", ProductionPointSellingFix)
end

function ProductionPointSellingFix:onLoad(savegame)
    local spec = self.spec_productionPoint
    print("onLoad")
    print("isOwned: " .. tostring(spec.productionPoint.isOwned))
    print("farmId: " .. tostring(spec.productionPoint.owningPlaceable:getOwnerFarmId()))

    spec.productionPoint.unloadingStation.storeSoldGoods = spec.productionPoint.unloadingStation.skipSell
end

function ProductionPointSellingFix:onBuy()
    local spec = self.spec_productionPoint

    spec.productionPoint.unloadingStation.storeSoldGoods = true
end

function ProductionPointSellingFix:onSell()
    local spec = self.spec_productionPoint

    spec.productionPoint.unloadingStation.storeSoldGoods = false
end

function ProductionPointSellingFix:buyRequest(superFunc)
	local storeItem = g_storeManager:getItemByXMLFilename(self.owningPlaceable.configFileName)
	local price = g_currentMission.economyManager:getBuyPrice(storeItem) or self.owningPlaceable:getPrice()

	if self.owningPlaceable.buysFarmland and self.owningPlaceable.farmlandId ~= nil then
		local farmland = g_farmlandManager:getFarmlandById(self.owningPlaceable.farmlandId)

		if farmland ~= nil and g_farmlandManager:getFarmlandOwner(self.owningPlaceable.farmlandId) ~= self.mission:getFarmId() then
			price = price + farmland.price
		end
	end

	local activatable = self.activatable
	local productionPoint = self

	local function buyingEventCallback(statusCode)
		if statusCode ~= nil then
			local dialogArgs = BuyExistingPlaceableEvent.DIALOG_MESSAGES[statusCode]

			if dialogArgs ~= nil then
				g_gui:showInfoDialog({
					text = g_i18n:getText(dialogArgs.text),
					dialogType = dialogArgs.dialogType
				})
			end
		end

		g_messageCenter:unsubscribe(BuyExistingPlaceableEvent, productionPoint)
		activatable:updateText()
	end

	local text = string.format(g_i18n:getText("dialog_buyBuildingFor"), self:getName(), g_i18n:formatMoney(price, 0, true))

	local function dialogCallback(yes)
		if yes then
			g_messageCenter:subscribe(BuyExistingPlaceableEvent, buyingEventCallback)
			g_client:getServerConnection():sendEvent(BuyExistingPlaceableEvent.new(self.owningPlaceable, self.mission:getFarmId()))
            self.unloadingStation.storeSoldGoods = true

		end
	end

	g_gui:showYesNoDialog({
		text = text,
		callback = dialogCallback
	})
end

PlaceableProductionPoint.registerEventListeners = Utils.appendedFunction(PlaceableProductionPoint.registerEventListeners, ProductionPointSellingFix.registerEventListeners)
PlaceableProductionPoint.onLoad = Utils.appendedFunction(PlaceableProductionPoint.onLoad, ProductionPointSellingFix.onLoad)
ProductionPoint.buyRequest = Utils.overwrittenFunction(ProductionPoint.buyRequest, ProductionPointSellingFix.buyRequest)